import unittest
from click import testing as click_testing
from .._cli.main import pysr


class TestCli(unittest.TestCase):
    # TODO: Include test for custom project here.
    def setUp(self):
        self.cli_runner = click_testing.CliRunner()

    def test_help_on_all_commands(self):
        expected = "\n".join(
            [
                "Usage: pysr [OPTIONS] COMMAND [ARGS]...",
                "",
                "Options:",
                "  --help  Show this message and exit.",
                "",
                "Commands:",
                "  install  Install Julia dependencies for PySR.",
                "",
            ]
        )
        result = self.cli_runner.invoke(pysr, ["--help"])
        self.assertEqual(expected, result.output)
        self.assertEqual(0, result.exit_code)

    def test_help_on_install(self):
        expected = "\n".join(
            [
                "Usage: pysr install [OPTIONS]",
                "",
                "  Install Julia dependencies for PySR.",
                "",
                "Options:",
                "  -p, --project PROJECT_DIRECTORY",
                "                                  Install in a specific Julia project (e.g., a",
                "                                  local copy of SymbolicRegression.jl).",
                "  -q, --quiet                     Disable logging.",
                "  --precompile                    Force precompilation of Julia libraries.",
                "  --no-precompile                 Disable precompilation.",
                "  --help                          Show this message and exit.",
                "",
            ]
        )
        result = self.cli_runner.invoke(pysr, ["install", "--help"])
        self.assertEqual(expected, result.output)
        self.assertEqual(0, result.exit_code)


def runtests():
    """Run all tests in cliTest.py."""
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTests(loader.loadTestsFromTestCase(TestCli))
    runner = unittest.TextTestRunner()
    return runner.run(suite)



from .test import runtests
from .test_env import runtests as runtests_env
from .test_jax import runtests as runtests_jax
from .test_torch import runtests as runtests_torch
from .test_cli import runtests as runtests_cli



"""CLI for running PySR's test suite."""
import argparse
import os

from . import *

if __name__ == "__main__":
    # Get args:
    parser = argparse.ArgumentParser()
    parser.usage = "python -m pysr.test [tests...]"
    parser.add_argument(
        "test",
        nargs="*",
        help="Test to run. One or more of 'main', 'env', 'jax', 'torch', 'cli'.",
    )

    # Parse args:
    args = parser.parse_args()
    tests = args.test

    if len(tests) == 0:
        # Raise help message:
        parser.print_help()
        raise SystemExit(1)

    # Run tests:
    for test in tests:
        if test in {"main", "env", "jax", "torch", "cli"}:
            cur_dir = os.path.dirname(os.path.abspath(__file__))
            print(f"Running test from {cur_dir}")
            if test == "main":
                runtests()
            elif test == "env":
                runtests_env()
            elif test == "jax":
                runtests_jax()
            elif test == "torch":
                runtests_torch()
            elif test == "cli":
                runtests_cli()
        else:
            parser.print_help()
            raise SystemExit(1)
